/*****************************************************************************
 * Apci32w.cs
 *
 * aPCI-P32
 * Copyright (C) ADTEK SYSTEM SCIENCE Co.,Ltd.
 *****************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;

namespace SampleP32
{
	/// <summary>
	/// Apci32w ̊Tv̐łB
	/// </summary>
	public class Apci32w
	{
		private int mErrWradll = SUCCESS;	// bp[֐G[
	
		/*==================*/
		/*	Error Code		*/
		/*==================*/
		public const int	SUCCESS					= 0;	/* Successfully proccessed */
		public const int	ERR_SYSTEM				= 1;	/* Call GetLastError() */
		public const int	ERR_NO_DEVICE			= 2;	/* No card */
		public const int	ERR_IN_USE				= 3;	/* Device is in use */
		public const int	ERR_INVALID_SLOT		= 4;	/* Invalid slot number */
		public const int	ERR_RESOURCE			= 5;	/* Resource exhaustion */
		public const int	ERR_INVALID_PORT		= 6;	/* Invalid port */
		public const int	ERR_INVALID_ARGUMENT	= 7;	/* Invalid argument */
		public const int	ERR_NO_CREATE			= 8;	/* No created device */
		public const int	ERR_WRAPDLL				= -1;	// bp[֐G[

		/*==========================================*/
		/*	aPCI-P32 card's relative port address	*/
		/*==========================================*/
		public const int	PORT_1					= 0;
		public const int	PORT_2					= 1;
		public const int	PORT_3					= 2;
		public const int	PORT_4					= 3;
		public const int	PORT_5					= 4;
		public const int	PORT_6					= 5;
		public const int	PORT_7					= 6;
		public const int	PORT_8					= 7;

		/*==================*/
		/*	Miscellaneous	*/
		/*==================*/
		public const int	MAX_SLOTS				= 16;	/* Maximum number of slots */
		public const int	MAX_PORTS				= 4;	/* Maximum number of user ports */

		public const short	SLOT_AUTO				= -1;	/* default search */

		/*==================================*/
		/*	Resource infomation structure	*/
		/*==================================*/
		public const int	MAX_MEM					= 9;
		public const int	MAX_IO					= 20;
		public const int	MAX_IRQ					= 7;
		public const int	MAX_DMA					= 7;
		public const int	MAX_RSV					= 3;

		[StructLayout(LayoutKind.Sequential)]
		public struct Resource
		{
			public int		dwNumMemWindows;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_MEM)]
			public int[]	dwMemBase;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_MEM)]
			public int[]	dwMemLength;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_MEM)]
			public int[]	dwMemAttrib;
			public int		dwNumIOPorts;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_IO)]
			public int[]	dwIOPortBase;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_IO)]
			public int[]	dwIOPortLength;
			public int		dwNumIRQs;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_IRQ)]
			public int[]	dwIRQRegisters;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_IRQ)]
			public int[]	dwIRQAttrib;
			public int		dwNumDMAs;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_DMA)]
			public int[]	dwDMALst;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_DMA)]
			public int[]	dwDMAAttrib;
			[MarshalAs(UnmanagedType.ByValArray,SizeConst = MAX_RSV)]
			public int[]	dwReserved1;
		}

		/*==================*/
		/*	API Functions	*/
		/*==================*/
		[DllImport("apci32.dll")]
		public extern static unsafe bool Apci32GetVersion( int* pdwDllVersion, int* pdwDriverVersion );
		[DllImport("apci32.dll")]
		public extern static unsafe bool Apci32Create( short* pwLogSlot );
		[DllImport("apci32.dll")]
		public extern static        bool Apci32Close( short wLogSlot );
		[DllImport("apci32.dll")]
		public extern static        bool Apci32GetResource( short wLogSlot, ref Resource pres );
		[DllImport("apci32.dll")]
		public extern static unsafe bool Apci32GetSwitchValue( short wLogSlot, int* pdwSwitchValue );
		[DllImport("apci32.dll")]
		public extern static unsafe bool Apci32InPort( short wLogSlot, int dwLogPort, byte* pbInValue );
		[DllImport("apci32.dll")]
		public extern static        int  Apci32GetLastError( short wLogSlot );

		public Apci32w()
		{
			// 
			// TODO: RXgN^ WbNɒǉĂB
			//
		}

		/*============================================================================
		// 	  aPCI-P32 DLL,޲ײްްޮݔԍ擾
		// 	  pdwDllVersion		DLLްޮݔԍi[̈ւ߲
		//		  pdwDriverVersion	޲ײްްޮݔԍi[̈ւ߲
		// ߂l  true
		//		  s false
		//==========================================================================*/
		public unsafe bool GetVersion(int* pdwDllVersion, int* pdwDriverVersion)
		{
			try {
				return Apci32GetVersion(pdwDllVersion, pdwDriverVersion);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return false;
			}
		}
		/*============================================================================
		// 	  aPCI-P32̔Cӂ޲̎gp錾
		// 	  pwLogSlot	 gp޲̘_ۯĔԍi[߲
		// ߂l  true
		//		  s false
		//==========================================================================*/
		public unsafe bool Create(short* pwLogSlot)
		{
			try {
				return Apci32Create(pwLogSlot);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return false;
			}
		}
		/*============================================================================
		// 	  aPCI-P32̔Cӂ޲̎gpJ
		// 	  wLogSlot ޲̘_ۯĔԍ
		// ߂l  true
		//		  s false
		//==========================================================================*/
		public bool Close(short wLogSlot)
		{
			try {
				return Apci32Close(wLogSlot);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return false;
			}
		}
		/*============================================================================
		// 	  w޲ؿ擾
		// 	  wLogSlot	ؿ擾޲̘_ۯĔԍ
		//		  pResInfo	ؿi[\߲̂
		// ߂l  true
		//		  s false
		//==========================================================================*/
		public bool GetResource(short wLogSlot, ref Resource pResInfo)
		{
			try {
				return Apci32GetResource(wLogSlot, ref pResInfo);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return false;
			}
		}
		/*============================================================================
		// 	  Cӂ߰Ăް
		// 	  wLogSlot	͂޲̘_ۯĔԍ
		//		  dwLogPort	͂_߰
		//		  pbInValue ްi[̈ւ߲
		// ߂l  true
		//		  s false
		//==========================================================================*/
		public unsafe bool InPort(short wLogSlot, int dwLogPort, byte* pbInValue)
		{
			try {
				return Apci32InPort(wLogSlot, dwLogPort, pbInValue);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return false;
			}
		}
		/*============================================================================
		// 	  aPCI-P32̴װ
		// 	  wLogSlot	װ޲̘_ۯĔԍ
		// ߂l װ
		//==========================================================================*/
		public int GetLastError(short wLogSlot)
		{
			try
			{
				if (mErrWradll == ERR_WRAPDLL)	// bp[֐G[Ȃ
				{
					mErrWradll = SUCCESS;		// NA
					return ERR_WRAPDLL;
				} 
				else 
				{
					return Apci32GetLastError(wLogSlot);
				}
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return ERR_WRAPDLL;
			}
		}
		/*============================================================================
		// 	  ۰ذ̒l擾
		//    wLogSlot			۰ذ̒l擾޲̘_ۯĔԍ
		// 		  pdwSwitchValue	۰ذ̒li[̈ւ߲
		// ߂l  true
		//		  s false
		//==========================================================================*/
		public unsafe bool GetSwitchValue(short wLogSlot, int* pdwSwitchValue)
		{
			try {
				return Apci32GetSwitchValue(wLogSlot, pdwSwitchValue);
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message, "aPCI-P32 Error", 
								MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				mErrWradll = ERR_WRAPDLL;		// bp[֐G[
				return false;
			}
		}

	}
}
